@@ -23,7 +23,6 @@
 #include "kateapp.h"
 #include "katemainwindow.h"
 #include "kateviewmanager.h"
-#include "katedocmanageradaptor.h"
 #include "katecontainer.h"
 #include "katesavemodifieddialog.h"
 
@@ -76,15 +75,12 @@ KateDocManager::KateDocManager (QObject *parent)
   } else {
     kDebug()<<"Editor does not support the container interface";
   }
+  
   // read in editor config
   m_editor->readConfig(KGlobal::config().data());
 
   m_documentManager = new Kate::DocumentManager (this);
 
-  ( void ) new KateDocManagerAdaptor( this );
-  m_dbusObjectPath = "/DocumentManager";
-  QDBusConnection::sessionBus().registerObject( m_dbusObjectPath, this );
-
   m_metaInfos = new KConfig("metainfos", KConfig::NoGlobals, "appdata" );
 
   createDoc ();
@@ -290,7 +286,7 @@ KTextEditor::Document *KateDocManager::openUrl (const KUrl& url, const QString &
   return doc;
 }
 
-bool KateDocManager::closeDocument(class KTextEditor::Document *doc, bool closeUrl)
+bool KateDocManager::closeDocument(KTextEditor::Document *doc, bool closeUrl)
 {
   if (!doc) return false;
 
