@@ -29,7 +29,6 @@
 #include <QtCore/QStack>
 
 #include <kmimetype.h>
-#include <kencodingprober.h>
 
 #include <ktexteditor/document.h>
 #include <ktexteditor/sessionconfiginterface.h>
@@ -47,7 +46,6 @@
 #include "katetextline.h"
 #include "katetextcursor.h"
 #include "katetextrange.h"
-#include "katecodefolding.h"
 #include "messageinterface.h"
 
 namespace KTextEditor {
@@ -60,10 +58,8 @@ namespace KIO { class TransferJob; }
 
 namespace Kate { class SwapFile; }
 
-class KateCodeFoldingTree;
 class KateBuffer;
 class KateView;
-class KateLineInfo;
 class KateDocumentConfig;
 class KateHighlighting;
 class KateUndoManager;
@@ -105,6 +101,7 @@ class KATEPART_TESTS_EXPORT KateDocument : public KTextEditor::Document,
   Q_INTERFACES(KTextEditor::MessageInterface)
 
   friend class KateDocumentTest;
+  friend class KateBuffer;
 
   public:
     explicit KateDocument (bool bSingleViewMode=false, bool bBrowserView=false, bool bReadOnly=false,
@@ -193,7 +190,6 @@ Q_SIGNALS:
     virtual QChar character(const KTextEditor::Cursor& position) const;
     virtual int lines() const;
     virtual KTextEditor::Cursor documentEnd() const;
-    int numVisLines() const;
     virtual int totalCharacters() const;
     virtual int lineLength(int line) const;
 
@@ -323,6 +319,7 @@ Q_SIGNALS:
     int editSessionNumber;
     QStack<int> editStateStack;
     bool editIsRunning;
+    bool m_undoMergeAllEdits;
 
   //
   // KTextEditor::UndoInterface stuff
@@ -695,16 +692,8 @@ Q_SIGNALS:
 
     KateHighlighting *highlight () const;
 
-  public Q_SLOTS:    //please keep prototypes and implementations in same order
-    void tagLines(int start, int end);
-    void tagLines(KTextEditor::Cursor start, KTextEditor::Cursor end);
-
-  //export feature, obsolute
   public Q_SLOTS:
-     void exportAs(const QString&) { }
-
-  Q_SIGNALS:
-    void preHighlightChanged(uint);
+    void tagLines(int start, int end);
 
   private Q_SLOTS:
     void internalHlChanged();
@@ -834,11 +823,6 @@ Q_SIGNALS:
   private:
     void updateDocName ();
 
-  public:
-    void lineInfo (KateLineInfo *info, int line) const;
-
-    KateCodeFoldingTree *foldingTree ();
-
   public:
     /**
      * @return whether the document is modified on disk since last saved
@@ -905,14 +889,10 @@ Q_SIGNALS:
 
    // code folding
   public:
-    uint getRealLine(unsigned int virtualLine);
-    uint getVirtualLine(unsigned int realLine);
-    uint visibleLines ();
     Kate::TextLine kateTextLine(uint i);
     Kate::TextLine plainKateTextLine(uint i);
 
   Q_SIGNALS:
-    void codeFoldingUpdated();
     void aboutToRemoveText(const KTextEditor::Range&);
 
   private Q_SLOTS:
@@ -1118,9 +1098,6 @@ Q_SIGNALS:
     QHash<KTextEditor::Message *, QList<QSharedPointer<QAction> > > m_messageHash;
   //END KTextEditor::MessageInterface
 
-  protected Q_SLOTS:
-      void dumpRegionTree();
-
   public:
       QString defaultDictionary() const;
       QList<QPair<KTextEditor::MovingRange*, QString> > dictionaryRanges() const;
@@ -1135,11 +1112,9 @@ Q_SIGNALS:
       void revertToDefaultDictionary(const KTextEditor::Range &range);
       void setDefaultDictionary(const QString& dict);
       void onTheFlySpellCheckingEnabled(bool enable);
-      void respellCheckBlock(int start, int end) {respellCheckBlock(this,start,end);}
       void refreshOnTheFlyCheck(const KTextEditor::Range &range = KTextEditor::Range::invalid());
 
   Q_SIGNALS:
-      void respellCheckBlock(KateDocument *document,int start, int end);
       void dictionaryRangesPresent(bool yesNo);
       void defaultDictionaryChanged(KateDocument *document);
 
