@@ -25,16 +25,13 @@
 #include <QtCore/QList>
 #include <QtGui/QAction>
 
+#include "katepartprivate_export.h"
+
 namespace KTextEditor {
 
 class View;
 class Document;
 
-//
-// TODOs and Ideas:
-// - turn FloatInView into TopRightInView, BottomRightInView, or Qt::Alignment?
-//
-
 /**
  * @brief This class holds a Message to display in View%s.
  *
@@ -90,7 +87,7 @@ class Document;
  * @author Dominik Haumann \<dhaumann@kde.org\>
  * @since KDE 4.10
  */
-class Message : public QObject
+class KATEPART_TESTS_EXPORT Message : public QObject
 {
   Q_OBJECT
 
@@ -117,7 +114,8 @@ class Message : public QObject
     enum MessagePosition {
       AboveView = 0, ///< show message above view
       BelowView,     ///< show message below view
-      FloatInView    ///< show message as view overlay
+      TopInView,     ///< show message as view overlay in the top right corner
+      BottomInView   ///< show message as view overlay om the bottom right corner
     };
 
     /**
@@ -200,7 +198,7 @@ class Message : public QObject
      */
     int autoHide() const;
 
-     /**
+    /**
      * Sets the autoHide mode to @p mode.
      * The default mode is set to AutoHideMode::AfterUserInteraction.
      * @param mode autoHide mode
@@ -219,8 +217,8 @@ class Message : public QObject
      * Enabled word wrap according to @p wordWrap.
      * By default, auto wrap is disabled.
      *
-     * If the text of the message is long, always enable auto wrap, as
-     * otherwise the layout of the gui breaks.
+     * Word wrap is enabled automatically, if the Message's width is larger than
+     * the parent widget's width to avoid breaking the gui layout.
      *
      * @see wordWrap()
      */
