@@ -2,7 +2,7 @@
  * name: C++/boost Style
  * license: LGPL
  * author: Alex Turbov <i.zaufi@gmail.com>
- * revision: 2
+ * revision: 6
  * kate-version: 3.4
  * priority: 10
  * indent-languages: C++11, C++11/Qt4
@@ -36,7 +36,6 @@
  * space-indent true;
  * auto-brackets true;
  * replace-tabs true;
- * replace-tabs true;
  * replace-tabs-save true;
  *
  * \todo Better to check (assert) some of that modelines...
@@ -55,9 +54,9 @@ require ("string.js");
 // ')' is for align dangling close bracket
 // ';' is for align `for' parts
 // TBD <others>
-triggerCharacters = "{}()<>/:;,#\\?|/%.";
+triggerCharacters = "{}()<>/:;,#\\?|/%.@";
 
-var debugMode = true;
+var debugMode = false;
 
 /// \todo Move to a separate library?
 function dbg()
@@ -94,6 +93,10 @@ function isInsideBraces(line, column, ch)
  *   \li \c hasComment -- boolean: \c true if comment present on the line, \c false otherwise
  *   \li \c before -- text before the comment
  *   \li \c after -- text of the comment
+ *
+ * \todo Make it smart and check highlighting style where \c '//' string is found.
+ * \todo Possible it would be quite reasonable to analyze a type of the comment:
+ * Is it C++ or Doxygen? Is it single or w/ some text before?
  */
 function splitByComment(text)
 {
@@ -110,19 +113,25 @@ function splitByComment(text)
     return {hasComment: found, before: before, after: after};
 }
 
+/**
+ * \brief Remove possible comment from text
+ */
 function stripComment(text)
 {
-    return splitByComment(text).before.rstrip();
+    var result = splitByComment(text);
+    if (result.hasComment)
+        return result.before.rtrim();
+    return text.rtrim();
 }
 
-/// Return \c true if attribute at given position is not a \e String or \e Comment
-function isNotStringOrComment(line, column)
+/// Return \c true if attribute at given position is a \e String or \e Comment
+function isStringOrComment(line, column)
 {
     // Check if we are not withning a string or a comment
     var c = new Cursor(line, column);
     var mode = document.attributeName(c);
-    dbg("isNotStringOrComment: Check mode @ " + c + ": " + mode);
-    return !(document.isString(c) || document.isComment(c));
+    dbg("isStringOrComment: Check mode @ " + c + ": " + mode);
+    return gMode == "Doxygen" || document.isString(c) || document.isComment(c);
 }
 
 /// Try to (re)align (to 60th position) inline comment if present
@@ -132,7 +141,7 @@ function alignInlineComment(line)
     var currentLineText = document.line(line);
     var sc = splitByComment(currentLineText);
     // Did we found smth and if so, make sure it is not a string or comment...
-    if (sc.hasComment && isNotStringOrComment(line, sc.before.length - 1))
+    if (sc.hasComment && !isStringOrComment(line, sc.before.length - 1))
     {
         var rbefore = sc.before.rtrim();
         /// \attention Kate has a BUG: even if everything is Ok and no realign
@@ -176,22 +185,48 @@ function alignInlineComment(line)
  */
 function tryToKeepInlineComment(line)
 {
+    // Make sure that there is some text still present on a prev line
+    // i.e. it was just splitted and same-line-comment must be moved back to it
+    if (document.line(line - 1).trim().length == 0)
+        return;
+
     // Check is there any comment on the current line
     var currentLineText = document.line(line);
     var sc = splitByComment(currentLineText);
-    if (sc.hasComment && isNotStringOrComment(line, sc.before.length - 1) && sc.before.length > 0)
+    if (sc.hasComment && !isStringOrComment(line, sc.before.length - 1) && sc.after.length > 0)
     {
-        // Yep, try to move it on a previous line.
-        // NOTE The latter can't have a comment!
-        var lastPos = document.lastColumn(line - 1);
-        document.insertText(
-            line - 1
-          , lastPos + 1
-          , String().fill(' ', gSameLineCommentStartAt - lastPos - 1)
-              + "//"
-              + sc.after.rtrim()
-          );
-        document.removeText(line, sc.before.rtrim().length, line, currentLineText.length);
+        // Ok, here is few cases possible when ENTER pressed in different positions
+        // |  |smth|was here; |        |// comment
+        //
+        // If sc.before has some text, it means that cursor was in the middle of some
+        // non-commented text, and part of it left on a prev line, so we have to move
+        // the comment back to that line...
+        if (sc.before.trim().length > 0)                    // Is there some text before comment?
+        {
+            var lastPos = document.lastColumn(line - 1);    // Get last position of non space char @ prev line
+            // Put the comment text to the prev line w/ padding
+            document.insertText(
+                line - 1
+              , lastPos + 1
+              , String().fill(' ', gSameLineCommentStartAt - lastPos - 1)
+                  + "//"
+                  + sc.after.rtrim()
+              );
+            // Remove it from current line starting from current position
+            // 'till the line end
+            document.removeText(line, sc.before.rtrim().length, line, currentLineText.length);
+        }
+        else
+        {
+            // No text before comment. Need to remove possible spaces from prev line...
+            var prevLine = line - 1;
+            document.removeText(
+                prevLine
+              , document.lastColumn(prevLine) + 1
+              , prevLine
+              , document.lineLength(prevLine)
+              );
+        }
     }
 }
 
@@ -280,8 +315,17 @@ function tryToAlignAfterOpenBrace_ch(line)
     var pos = document.lastColumn(line - 1);
     var ch = document.charAt(line - 1, pos);
 
-    if (ch == '{' || ch == '(' || ch == '[')
+    if (ch == '(' || ch == '[')
+    {
         result = document.firstColumn(line - 1) + gIndentWidth;
+    }
+    else if (ch == '{')
+    {
+        if (document.startsWith(line - 1, "namespace", true))
+            result = 0;
+        else
+            result = document.firstColumn(line - 1) + gIndentWidth;
+    }
     else if (ch == '<')
     {
         // Does it looks like 'operator<<'?
@@ -354,7 +398,8 @@ function tryMultilineCommentStart_ch(line)
 {
     var result = -1;
     // Check if multiline comment was started on the line
-    if (document.startsWith(line - 1, "/*", true))
+    // and ENTER wan't pressed right after a /*C-style comment*/
+    if (document.startsWith(line - 1, "/*", true) && !document.endsWith(line - 1, "*/", true))
     {
         var filler = String().fill(' ', document.firstVirtualColumn(line - 1) + 1);
         var padding = filler + "* ";
@@ -362,17 +407,36 @@ function tryMultilineCommentStart_ch(line)
         // then append a comment closer also...
         if ((line + 1) < document.lines())
         {
+            // Maybe user wants to extend a multiline C-style/Doxygen comment
+            // by pressing ENTER at start of it?
             if (!document.startsWith(line + 1, "*", true))
-                padding += "\n" + filler + "*/";
+            {
+                // ... doesn't looks like a multiline comment
+                padding += "\n" + filler;
+                // Maybe user just splits a C-style comment?
+                if (!document.startsWith(line, "*/", true))
+                    padding += document.endsWith(line, "*/", true) ? "* " : "*/";
+                else
+                    document.removeText(line, 0, line, document.firstColumn(line))
+            }                                               // else, no need to append a closing */
+        }
+        else                                                // There is no a next line...
+        {
+            padding += "\n" + filler;
+            if (!document.startsWith(line, "*/", true))
+                padding += document.endsWith(line, "*/", true) ? "* " : "*/";
+            else
+                document.removeText(line, 0, line, document.firstColumn(line))
         }
-        else padding += "\n" + filler + "*/\n";             // There is no a next line...
 
         document.insertText(line, 0, padding);
-        view.setCursorPosition(line, padding.length + 4);
-        result = filler.length;
+        view.setCursorPosition(line, filler.length + 2);
+        result = -2;
     }
     if (result != -1)
+    {
         dbg("tryMultilineCommentStart_ch result="+result);
+    }
     return result;
 }
 
@@ -402,7 +466,23 @@ function tryMultilineCommentCont_ch(line)
         }
     }
     if (result != -1)
+    {
         dbg("tryMultilineCommentCont_ch result="+result);
+    }
+    return result;
+}
+
+function tryAfterCloseMultilineComment_ch(line)
+{
+    var result = -1;
+    if (document.startsWith(line - 1, "*/", true))
+    {
+        result = document.firstColumn(line - 1) - 1;
+    }
+    if (result != -1)
+    {
+        dbg("tryAfterCloseMultilineComment_ch result="+result);
+    }
     return result;
 }
 
@@ -441,7 +521,9 @@ function trySplitComment_ch(line)
         }
     }
     if (result != -1)
+    {
         dbg("trySplitComment_ch result="+result);
+    }
     return result;
 }
 
@@ -451,13 +533,36 @@ function tryIndentAfterSomeKeywords_ch(line)
     var result = -1;
     // Check if ENTER was pressed after some keywords...
     var prevString = document.line(line - 1);
-    var r = /^(\s*)((if|for|while)\s*\(|\bdo\b|\belse\b|(public|protected|private|default|case\s+.*)\s*:).*$/
+    var r = /^(\s*)((if|for|while)\s*\(|\bdo\b|(public|protected|private|default|case\s+.*)\s*:).*$/
       .exec(prevString);
     if (r != null)
     {
         dbg("r=",r);
         result = r[1].length + gIndentWidth;
     }
+    else
+    {
+        r = /^\s*\belse\b.*$/.exec(prevString)
+        if (r != null)
+        {
+            var prevPrevString = document.line(line - 2);
+            prevPrevString = stripComment(prevPrevString);
+            dbg("tryIndentAfterSomeKeywords_ch prevPrevString="+prevPrevString);
+            if (prevPrevString.endsWith('}'))
+                result = document.firstColumn(line - 2);
+            else if (prevPrevString.match(/^\s*[\])>]/))
+                result = document.firstColumn(line - 2) - gIndentWidth - (gIndentWidth/2);
+            else
+                result = document.firstColumn(line - 2) - gIndentWidth;
+            // Realign 'else' statement if needed
+            var pp = document.firstColumn(line - 1);
+            if (pp < result)
+                document.insertText(line - 1, 0, String().fill(' ', result - pp));
+            else if (result < pp)
+                document.removeText(line - 1, 0, line - 1, pp - result);
+            result += gIndentWidth;
+        }
+    }
     if (result != -1)
     {
         tryToKeepInlineComment(line);
@@ -519,7 +624,30 @@ function tryMacroDefinition_ch(line)
     if (prevString.search(/^\s*#\s*define\s+.*\\$/) != -1)
         result = gIndentWidth;
     if (result != -1)
+    {
         dbg("tryMacroDefinition_ch result="+result);
+    }
+    return result;
+}
+
+/**
+ * Do not incrase indent if ENTER pressed before access
+ * specifier (i.e. public/private/protected)
+ */
+function tryBeforeAccessSpecifier_ch(line)
+{
+    var result = -1;
+    if (document.line(line).match(/(public|protected|private):/))
+    {
+        var openPos = document.anchor(line, 0, '{');
+        if (openPos.isValid())
+            result = document.firstColumn(openPos.line);
+    }
+    if (result != -1)
+    {
+        tryToKeepInlineComment(line);
+        dbg("tryBeforeAccessSpecifier_ch result="+result);
+    }
     return result;
 }
 
@@ -530,7 +658,6 @@ function tryMacroDefinition_ch(line)
 function tryBeforeDanglingDelimiter_ch(line)
 {
     var result = -1;
-    dbg("text='"+document.line(line)+"'");
     var halfTabNeeded =
         // current line do not starts w/ a comment
         !document.line(line).ltrim().startsWith("//")
@@ -593,7 +720,9 @@ function tryPreprocessor_ch(line)
         }
     }
     if (result != -1)
+    {
         dbg("tryPreprocessor_ch result="+result);
+    }
     return result;
 }
 
@@ -621,6 +750,7 @@ function caretPressed(cursor)
         tryBraceSplit_ch                                    // Handle ENTER between braces
       , tryMultilineCommentStart_ch
       , tryMultilineCommentCont_ch
+      , tryAfterCloseMultilineComment_ch
       , trySplitComment_ch
       , tryToAlignAfterOpenBrace_ch                         // Handle {,[,(,< on a previous line
       , tryToAlignBeforeCloseBrace_ch                       // Handle },],),> on a current line before cursor
@@ -629,6 +759,7 @@ function caretPressed(cursor)
       , tryAfterDanglingSemicolon_ch
       , tryMacroDefinition_ch
       , tryBeforeDanglingDelimiter_ch
+      , tryBeforeAccessSpecifier_ch
       , tryAfterEqualChar_ch
       , tryPreprocessor_ch
       , tryToKeepInlineComment_ch                           // NOTE This must be a last checker!
@@ -654,8 +785,11 @@ function caretPressed(cursor)
  *     position.
  * \li just entered \c '/' is a 3rd in a sequence. If there is some text before and no after,
  *     it looks like inlined doxygen comment, so append \c '<' char after. Do nothing otherwise.
+ * \li if there is a <tt>'// '</tt> string right before just entered \c '/', form a
+ *     doxygen comment <tt>'///'</tt> or <tt>'///<'</tt> depending on presence of some text
+ *     on a line before the comment.
  *
- * \todo Due a BUG in a current version of Kate, this code doesn't work as expected!
+ * \todo Due the BUG #316809 in a current version of Kate, this code doesn't work as expected!
  * It always returns a <em>"NormalText"</em>!
  * \code
  * var cm = document.attributeName(cursor);
@@ -675,38 +809,49 @@ function trySameLineComment(cursor)
     var line = cursor.line;
     var column = cursor.column;
 
-    // Fisrt of all check that we r not withing a string
-    // BUG See note in a function's documentation
-    var cm = document.attributeName(cursor);
-    if (cm.indexOf("String") != -1)
-       return;
+    // First of all check that we are not withing a string
+    if (document.isString(line, column)) {
+        return;
+    }
 
-    var match = /^([^\/]*)(\/\/+)(.*)$/.exec(document.line(line));
-
-    if (match != null)                                      // Is matched?
+    var sc = splitByComment(document.line(line));
+    if (sc.hasComment)                                      // Is there any comment on a line?
     {
-        dbg("match_before  = '" + match[1] + "'");
-        dbg("match_comment = '" + match[2] + "'");
-        dbg("match_after   = '" + match[3] + "'");
-        if (match[2] == "///" && match[3].trim().length == 0)
-        {
-            // 3rd case here!
-            var filler = (match[1].trim().length > 0)       // Is there any text before comment?
-                ? "< "                                      // turn it into inline-doxygen comment
-                : " ";                                      // just usual doxygen comment
-            document.insertText(cursor, filler);
+        // Make sure we r not in a comment already
+        if (document.isComment(line, document.firstColumn(line)) &&
+          (document.line(line) != '///')) {
+            return;
         }
-        else if (match[2] == "//" && 0 < match[1].trim().length && match[3].length == 0)
+        // If no text after the comment and it still not aligned
+        var text_len = sc.before.rtrim().length;
+        if (text_len != 0 && sc.after.length == 0 && text_len < gSameLineCommentStartAt)
         {
-            // 2nd case here! Check if padding required
-            if (match[1].length < gSameLineCommentStartAt)
-            {
-                var filler = String().fill(' ', gSameLineCommentStartAt - match[1].length) + "//";
-                document.editBegin();
-                document.removeText(line, column - 2, line, column);
-                document.insertText(line, column - 2, filler);
-                document.editEnd();
-            }
+            // Align it!
+            document.insertText(
+                line
+              , column - 2
+              , String().fill(' ', gSameLineCommentStartAt - text_len)
+              );
+            document.insertText(line, gSameLineCommentStartAt + 2, ' ');
+        }
+        // If text in a comment equals to '/' or ' /' -- it looks like a 3rd '/' pressed
+        else if (sc.after == " /" || sc.after == "/")
+        {
+            // Form a Doxygen comment!
+            document.removeText(line, column - sc.after.length, line, column);
+            document.insertText(line, column - sc.after.length, text_len != 0 ? "/< " : "/ ");
+        }
+        // If right trimmed text in a comment equals to '/' -- it seems user moves cursor
+        // one char left (through space) to add one more '/'
+        else if (sc.after.rtrim() == "/")
+        {
+            // Form a Doxygen comment!
+            document.removeText(line, column, line, column + sc.after.length);
+            document.insertText(line, column, text_len != 0 ? "< " : " ");
+        }
+        else if (text_len == 0 && sc.after.length == 0)
+        {
+            document.insertText(line, column, ' ');
         }
     }
 }
@@ -718,27 +863,46 @@ function trySameLineComment(cursor)
  * \li user entered <em>"template &gt;</em>
  * \li user entered smth like <em>std::map&gt;</em>
  * \li user wants to output smth to C++ I/O stream by typing <em>&gt;&gt;</em>
+ *
+ * But, do not add '>' if there some text after cursor.
  */
 function tryTemplate(cursor)
 {
     var line = cursor.line;
     var column = cursor.column;
 
+    if (isStringOrComment(line, column))
+        return;                                             // Do nothing for comments and strings
+
     // Check for 'template' keyword at line start
     var currentString = document.line(line);
     var prevWord = document.wordAt(line, column - 1);
-    var isCloseAngleBracketNeeded = currentString.match(/^\s*template\s*<$/)
-      || prevWord.match(/[A-Za-z_][A-Za-z0-9_]*/)           // Does a word before '<' looks like identifier?
+    dbg("tryTemplate: prevWord='"+prevWord+"'");
+    dbg("tryTemplate: prevWord.match="+prevWord.match(/\b[A-Za-z_][A-Za-z0-9_]*/));
+    // Add a closing angle bracket if a prev word is not a 'operator'
+    // and it looks like an identifier or current line starts w/ 'template' keyword
+    var isCloseAngleBracketNeeded = (prevWord != "operator")
+      && (currentString.match(/^\s*template\s*<$/) || prevWord.match(/\b[A-Za-z_][A-Za-z0-9_]*/))
+      && (column == document.lineLength(line) || document.charAt(cursor).match(/\W/))
       ;
     if (isCloseAngleBracketNeeded)
     {
         document.insertText(cursor, ">");
         view.setCursorPosition(cursor);
     }
+    // Add a space after 2nd '<' if a word before is not a 'operator'
     else if (document.charAt(line, column - 2) == '<')
     {
-        // Looks like case 3... add a space after operator<<
-        document.insertText(line, column, " ");
+        if (document.wordAt(line, column - 3) != "operator")
+        {
+            // Looks like case 3... add a space after operator<<
+            document.insertText(line, column, " ");
+        }
+        else
+        {
+            document.insertText(line, column, "()");
+            view.setCursorPosition(line, column + 1);
+        }
     }
 }
 
@@ -762,7 +926,10 @@ function tryComma(cursor)
         var mustMove = !(prevLineFirstChar == ',' || prevLineFirstChar == ':');
         result = document.firstColumn(line - 1) - (mustMove ? 2 : 0);
     }
-    document.insertText(cursor, " ");                       // Always add one space after comma
+    if (document.charAt(cursor) != ' ')
+        document.insertText(cursor, " ");                   // Add space only if not present
+    else
+        view.setCursorPosition(line, column + 1);           // Otherwise just move cursor after it
     return result;
 }
 
@@ -820,16 +987,68 @@ function tryCloseBracket(cursor, ch)
     var line = cursor.line;
     var column = cursor.column;
 
+    var braceCursor = Cursor.invalid();
+    if (ch != '>')
+    {
+        // TODO Make sure a given `ch` in the gBraceMap
+        braceCursor = document.anchor(line, column - 1, gBraceMap[ch]);
+        // TODO Otherwise, it seems we have a template parameters list...
+    }
+
     // Check if a given closing brace is a first char on a line
     // (i.e. it is 'dangling' brace)...
-    if (document.firstChar(line) == ch && document.firstColumn(line) == (column - 1))
+    if (document.firstChar(line) == ch && document.firstColumn(line) == (column - 1) && braceCursor.isValid())
     {
-        var braceCursor = new Cursor().invalid();
-        if (ch != '>')
-            braceCursor = document.anchor(line, column - 1, gBraceMap[ch]);
-            // TODO Otherwise, it seems we have a template parameters list...
-        if (braceCursor.isValid())
-            result = document.firstColumn(braceCursor.line);
+        // Move to one half-TAB right, if anything but not closing '}', else
+        // align to the corresponding open char
+        result = document.firstColumn(braceCursor.line) + (ch != '}' ? 2 : 0);
+        dbg("tryCloseBracket: setting result="+result);
+    }
+
+    // Check if ';' required after closing '}'
+    if (ch == '}' && braceCursor.isValid())
+    {
+        var is_check_needed = false;
+        // Check if corresponding anchor is a class/struct/union/enum,
+        // (possible keyword located on same or prev line)
+        // and check for trailing ';'...
+        var anchoredString = document.line(braceCursor.line);
+        dbg("tryCloseBracket: anchoredString='"+anchoredString+"'");
+        var regex = /^(\s*)(class|struct|union|enum).*$/;
+        var r = regex.exec(anchoredString);
+        if (r != null)
+        {
+            dbg("tryCloseBracket: same line");
+            is_check_needed = true;
+        }
+        else (!is_check_needed && 0 < braceCursor.line)     // Is there any line before?
+        {
+            dbg("tryCloseBracket: cheking prev line");
+
+            // Ok, lets check it!
+            anchoredString = document.line(braceCursor.line - 1);
+            dbg("tryCloseBracket: anchoredString-1='"+anchoredString+"'");
+            r = regex.exec(anchoredString);
+            if (r != null)
+            {
+                is_check_needed = true;
+                dbg("tryCloseBracket: prev line");
+            }
+        }
+        dbg("tryCloseBracket: is_check_needed="+is_check_needed);
+        if (is_check_needed)
+        {
+            var is_ok = document.line(line)
+              .substring(column, document.lineLength(line))
+              .ltrim()
+              .startsWith(';')
+              ;
+            if (!is_ok)
+            {
+                document.insertText(line, column, ';');
+                view.setCursorPosition(line, column + 1);
+            }
+        }
     }
 
     return result;
@@ -838,7 +1057,12 @@ function tryCloseBracket(cursor, ch)
 /**
  * \brief Indent new scope block
  *
- * ... try to unindent to be precise...
+ * ... try to unindent to be precise... First of all check that open
+ * \c '{' is a first symbol on a line, and if it doesn't, make sure
+ * there is a space before, except if previous char is not a \c '{'.
+ * Otherwise, look at the previous line for dangling <tt>')'</tt> or
+ * a line started w/ one of flow control keywords.
+ *
  */
 function tryBlock(cursor)
 {
@@ -846,18 +1070,37 @@ function tryBlock(cursor)
     var line = cursor.line;
     var column = cursor.column;
 
-    // Check for a dangling close brace on a previous line
-    // (this may mean that `for' or `if' or `while' w/ looong parameters list on it)
-    if (document.firstChar(line - 1) == ')')
-        result = Math.floor(document.firstColumn(line - 1) / gIndentWidth) * gIndentWidth;
+    // Make sure we r not in a comment or string
+    dbg("tryBlock: isStringOrComment(line, column - 2)="+isStringOrComment(line, column - 2))
+    if (isStringOrComment(line, column - 2))
+        return result;
+
+    if (document.firstColumn(line) == (column - 1) && document.firstChar(line) == '{')
+    {
+        // Check for a dangling close brace on a previous line
+        // (this may mean that `for' or `if' or `while' w/ looong parameters list on it)
+        if (document.firstChar(line - 1) == ')')
+            result = Math.floor(document.firstColumn(line - 1) / gIndentWidth) * gIndentWidth;
+        else
+        {
+            // Otherwise, check for a keyword on the previous line and
+            // indent the started block to it...
+            var prevString = document.line(line - 1);
+            var r = /^(\s*)((catch|if|for|while)\s*\(|do|else|try|(default|case\s+.*)\s*:).*$/.exec(prevString);
+            if (r != null)
+                result = r[1].length;
+        }
+    }
     else
     {
-        // Otherwise, check for a keyword on the previous line and
-        // indent the started block to it...
-        var prevString = document.line(line - 1);
-        var r = /^(\s*)((catch|if|for|while)\s*\(|do|else|try|(default|case\s+.*)\s*:).*$/.exec(prevString);
-        if (r != null)
-            result = r[1].length;
+        // '{' is not a first char. Check for previous one...
+        if (1 < column)
+        {
+            var prevChar = document.charAt(line, column - 2);
+            dbg("tryBlock: prevChar='"+prevChar+"'");
+            if (prevChar != ' ' && prevChar != '{' && prevChar != '(')
+                document.insertText(line, column - 1, ' ');
+        }
     }
     return result;
 }
@@ -915,13 +1158,33 @@ function tryColon(cursor)
     }
     else
     {
-        var currentString = document.line(line);
-        if (currentString.search(/^\s*((public|protected|private)\s*(slots|Q_SLOTS)?|(signals|Q_SIGNALS)\s*):\s*$/) != -1)
+        var currentLine = document.line(line);
+        if (currentLine.search(/^\s*((public|protected|private)\s*(slots|Q_SLOTS)?|(signals|Q_SIGNALS)\s*):\s*$/) != -1)
         {
             var definitionCursor = document.anchor(line, 0, '{');
             if (definitionCursor.isValid())
             {
                 result = document.firstVirtualColumn(definitionCursor.line);
+                dbg("tryColon: result="+result);
+                if (0 < line)                               // Is there any line before?
+                {
+                    // Check if previous line is not empty and not starts w/ '{'
+                    var prevLine = document.line(line - 1).trim()
+                    if (prevLine.length && !prevLine.startsWith("{"))
+                    {
+                        // Cuz a new line will be added in place of current, returning
+                        // result will not affect indentation. So do it manually.
+                        var firstColumn = document.firstColumn(line);
+                        var padding = "";
+                        if (firstColumn < result)
+                            padding = String().fill(' ', result - firstColumn);
+                        else if (result < firstColumn)
+                            document.removeText(line, 0, line, firstColumn - result);
+                        // Add an empty line before the current
+                        document.insertText(line, 0, "\n" + padding);
+                        result = 0;
+                    }
+                }
             }
         }
     }
@@ -937,7 +1200,7 @@ function tryOpenBrace(cursor)
     var column = cursor.column;
     var wordBefore = document.wordAt(line, column - 1);
     dbg("word before: '"+wordBefore+"'");
-    if (wordBefore.search(/\b(catch|for|if|switch|while)\b/) != -1)
+    if (wordBefore.search(/\b(catch|for|if|switch|while|return)\b/) != -1)
         document.insertText(line, column - 1, " ");
 }
 
@@ -1021,6 +1284,30 @@ function tryBackslash(cursor)
     }
 }
 
+/**
+ * \brief Handle a <tt>@</tt> symbol
+ *
+ * Possible user wants to add a Doxygen group
+ */
+function tryDoxygenGrouping(cursor)
+{
+    var line = cursor.line;
+    var column = cursor.column;
+    var firstColumn = document.firstColumn(line);
+    // Check the symbol before the just entered
+    var looks_like_doxgorup = isStringOrComment(line, column - 2)// ")
+      && firstColumn == (column - 4)
+      && document.line(line).ltrim().startsWith("// ")
+      ;
+    if (looks_like_doxgorup)
+    {
+        document.removeText(line, column - 2, line, column - 1);
+        var padding = String().fill(' ', firstColumn);
+        document.insertText(line, column - 1, "{\n" + padding + "\n" + padding + "//@}");
+        view.setCursorPosition(line + 1, document.lineLength(line + 1));
+    }
+}
+
 /**
  * \brief Process one character
  *
@@ -1063,7 +1350,7 @@ function processChar(line, ch)
         case '?':
         case '|':
         case '%':
-        case '/':
+        case '/':                                           // TODO Useless! Code review needed.
         case '.':
             result = tryOperator(cursor, ch);               // Possible need to align some operator
             break;
@@ -1088,6 +1375,9 @@ function processChar(line, ch)
         case '\\':
             tryBackslash(cursor);
             break;
+        case '@':
+            tryDoxygenGrouping(cursor);
+            break;
         default:
             break;                                          // Nothing to do...
     }
@@ -1155,9 +1445,11 @@ function findSingleLineCommentBlockEnd(line)
         if (text.length == 0) continue;                     // Skip empty lines...
         if (!text.startsWith("//")) break;                  // Yeah! Smth was found finally.
     }
+    var currentLineText = document.line(line).ltrim();      // Get text of the found line
+    while (currentLineText.length == 0)                     // Skip empty lines if any
+        currentLineText = document.line(line).ltrim();
     // Make sure it is not another one multiline comment, and if so,
     // going to find it's end as well...
-    var currentLineText = document.line(line).ltrim();
     if (currentLineText.startsWith("/*"))
         line = findMultiLineCommentBlockEnd(line);
     else if (document.lines() <= line)
@@ -1211,13 +1503,25 @@ function alignInsideBraces(line)
     // lets check that current line starts w/ '*' also!
     // NOTE Yep, it is expected (hardcoeded) that multiline comment has
     // all lines strarted w/ a star symbol!
-    else if (document.isComment(line, 0) && currentLineText.startsWith("*"))
+    // TODO BUG Kate has a bug: when multiline code snippet gets inserted into
+    // a multiline comment block (like Doxygen's @code/@endcode)
+    // document.isComment() returns true *only& for the first line of it!
+    // So some other way needs to be found to indent comments properly...
+    // TODO DAMN... it doesn't work that way also... for snippets longer than 2 lines.
+    // I suppose kate first insert text, then indent it, and after that highlight it
+    // So indenters based on a higlighting info will not work! BUT THEY DEFINITELY SHOULD!
+    else if (currentLineText.startsWith("*") && document.isComment(line, 0))
     {
         dbg("found middle of a multiline comment");
         // Yep, go to find a next non-comment line...
         nextNonCommentLine = findMultiLineCommentBlockEnd(line);
         middleOfMultilineBlock = true;
     }
+    dbg("line="+line);
+    dbg("document.isComment(line, 0)="+document.isComment(line, 0));
+    //dbg("document.defStyleNum(line, 0)="+document.defStyleNum(line-1, 0));
+    dbg("currentLineText='"+currentLineText+"'");
+    dbg("middleOfMultilineBlock="+middleOfMultilineBlock);
 
     if (nextNonCommentLine == 0)                            // End of comment not found?
         // ... possible due temporary invalid code...
@@ -1227,15 +1531,16 @@ function alignInsideBraces(line)
     if (nextNonCommentLine != -1)
     {
         // Yep, lets try to get desired indent for next non-comment line
-        dbg("** next non comment line is "+nextNonCommentLine);
-        dbg("** going to get its' indentation...");
         var desiredIndent = indentLine(nextNonCommentLine);
-        dbg("** got desired indentation for next non comment line: "+desiredIndent);
         if (desiredIndent < 0)
-            return desiredIndent;                           // Have no idea how to indent this comment!
+        {
+            // Have no idea how to indent this comment! So try to align it
+            // as found line:
+            desiredIndent = document.firstColumn(nextNonCommentLine);
+        }
         // TODO Make sure that next non-comment line do not starts
         // w/ 'special' chars...
-        return desiredIndent + middleOfMultilineBlock;
+        return desiredIndent + (middleOfMultilineBlock|0);
     }
 
     var brackets = [
@@ -1397,4 +1702,8 @@ function indent(line, indentWidth, ch)
     return indentLine(line);
 }
 
+/**
+ * \todo Better to use \c defStyleNum() instead of \c attributeName() and string comparision
+ */
+
 // kate: space-indent on; indent-width 4; replace-tabs on;
