@@ -232,7 +232,8 @@ void KateSearchBar::setReplacementPattern(const QString &replacementPattern) {
 
 
 
-QString KateSearchBar::replacementPattern() const {
+QString KateSearchBar::replacementPattern() const
+{
     Q_ASSERT(isPower());
 
     return m_powerUi->replacement->currentText();
@@ -240,7 +241,8 @@ QString KateSearchBar::replacementPattern() const {
 
 
 
-void KateSearchBar::setSearchMode(KateSearchBar::SearchMode mode) {
+void KateSearchBar::setSearchMode(KateSearchBar::SearchMode mode)
+{
     Q_ASSERT(isPower());
 
     m_powerUi->searchMode->setCurrentIndex(mode);
@@ -248,7 +250,8 @@ void KateSearchBar::setSearchMode(KateSearchBar::SearchMode mode) {
 
 
 
-void KateSearchBar::findNext() {
+void KateSearchBar::findNext()
+{
     const bool found = find();
 
     if (found) {
@@ -277,7 +280,7 @@ void KateSearchBar::showInfoMessage(const QString& text)
     delete m_infoMessage;
 
     m_infoMessage = new KTextEditor::Message(KTextEditor::Message::Positive, text);
-    m_infoMessage->setPosition(KTextEditor::Message::FloatInView);
+    m_infoMessage->setPosition(KTextEditor::Message::BottomInView);
     m_infoMessage->setAutoHide(3000); // 3 seconds
     m_infoMessage->setView(m_view);
 
@@ -531,6 +534,12 @@ bool KateSearchBar::find(SearchDirection searchDirection, const QString * replac
         return false; // == Pattern error
     }
 
+    // don't let selectionChanged signal mess around in this routine
+    disconnect(m_view, SIGNAL(selectionChanged(KTextEditor::View*)), this, SLOT(updateSelectionOnly()));
+
+    // clear previous highlights if there are any
+    clearHighlights();
+
     const Search::SearchOptions enabledOptions = searchOptions(searchDirection);
 
     // Where to find?
@@ -550,19 +559,19 @@ bool KateSearchBar::find(SearchDirection searchDirection, const QString * replac
         }
     } else {
         // No selection
-            const Cursor cursorPos = m_view->cursorPosition();
-            if (searchDirection == SearchForward) {
-                // if the vi input mode is used, the cursor will stay a the first character of the
-                // matched pattern (no selection will be made), so the next search should start from
-                // match column + 1
-                if (!m_view->viInputMode()) {
-                    inputRange.setRange(cursorPos, m_view->document()->documentEnd());
-                } else {
-                    inputRange.setRange(Cursor(cursorPos.line(), cursorPos.column()+1), m_view->document()->documentEnd());
-                }
+        const Cursor cursorPos = m_view->cursorPosition();
+        if (searchDirection == SearchForward) {
+            // if the vi input mode is used, the cursor will stay a the first character of the
+            // matched pattern (no selection will be made), so the next search should start from
+            // match column + 1
+            if (!m_view->viInputMode()) {
+                inputRange.setRange(cursorPos, m_view->document()->documentEnd());
             } else {
-                inputRange.setRange(Cursor(0, 0), cursorPos);
+                inputRange.setRange(Cursor(cursorPos.line(), cursorPos.column()+1), m_view->document()->documentEnd());
             }
+        } else {
+            inputRange.setRange(Cursor(0, 0), cursorPos);
+        }
     }
     FAST_DEBUG("Search range is" << inputRange);
 
@@ -582,30 +591,30 @@ bool KateSearchBar::find(SearchDirection searchDirection, const QString * replac
     // Find, first try
     match.searchText(inputRange, searchPattern());
     if (match.isValid() && match.range() == selection) {
-            // Same match again
-            if (replacement != 0) {
-                // Selection is match -> replace
-                KTextEditor::MovingRange *smartInputRange = m_view->doc()->newMovingRange (inputRange, KTextEditor::MovingRange::ExpandLeft | KTextEditor::MovingRange::ExpandRight);
-                afterReplace = match.replace(*replacement, m_view->blockSelection());
-                inputRange = *smartInputRange;
-                delete smartInputRange;
+        // Same match again
+        if (replacement != 0) {
+            // Selection is match -> replace
+            KTextEditor::MovingRange *smartInputRange = m_view->doc()->newMovingRange (inputRange, KTextEditor::MovingRange::ExpandLeft | KTextEditor::MovingRange::ExpandRight);
+            afterReplace = match.replace(*replacement, m_view->blockSelection());
+            inputRange = *smartInputRange;
+            delete smartInputRange;
+        }
+
+        if (!selectionOnly()) {
+            // Find, second try after old selection
+            if (searchDirection == SearchForward) {
+                const Cursor start = (replacement != 0) ? afterReplace.end() : selection.end();
+                inputRange.setRange(start, inputRange.end());
+            } else {
+                const Cursor end = (replacement != 0) ? afterReplace.start() : selection.start();
+                inputRange.setRange(inputRange.start(), end);
             }
+        }
 
-            if (!selectionOnly()) {
-                // Find, second try after old selection
-                if (searchDirection == SearchForward) {
-                    const Cursor start = (replacement != 0) ? afterReplace.end() : selection.end();
-                    inputRange.setRange(start, inputRange.end());
-                } else {
-                    const Cursor end = (replacement != 0) ? afterReplace.start() : selection.start();
-                    inputRange.setRange(inputRange.start(), end);
-                }
-            }
+        // Single-line pattern workaround
+        fixForSingleLine(inputRange, searchDirection);
 
-            // Single-line pattern workaround
-            fixForSingleLine(inputRange, searchDirection);
-
-            match.searchText(inputRange, searchPattern());
+        match.searchText(inputRange, searchPattern());
     }
 
     const bool wrap = !match.isValid() && (!selection.isValid() || !selectionOnly());
@@ -624,12 +633,14 @@ bool KateSearchBar::find(SearchDirection searchDirection, const QString * replac
                                                                        MatchWrappedBackward;
     indicateMatch(matchResult);
 
-    // Reset highlighting for all matches and highlight replacement if there is one
-    clearHighlights();
+    // highlight replacements if applicable
     if (afterReplace.isValid()) {
         highlightReplacement(afterReplace);
     }
 
+    // restore connection
+    connect(m_view, SIGNAL(selectionChanged(KTextEditor::View*)), this, SLOT(updateSelectionOnly()));
+    
     return true; // == No pattern error
 }
 
@@ -647,7 +658,8 @@ void KateSearchBar::findAll()
     const int occurrences = findAll(inputRange, NULL);
 
     // send passive notification to view
-    showInfoMessage(i18np("1 match found", "%1 matches found", occurrences));
+    showInfoMessage(i18ncp("short translation", "1 match found", "%1 matches found", occurrences));
+
     indicateMatch(occurrences > 0 ? MatchFound : MatchMismatch);
 }
 
@@ -779,6 +791,9 @@ void KateSearchBar::replaceNext() {
 // replacement != NULL --> Replace and highlight all matches
 int KateSearchBar::findAll(Range inputRange, const QString * replacement)
 {
+    // don't let selectionChanged signal mess around in this routine
+    disconnect(m_view, SIGNAL(selectionChanged(KTextEditor::View*)), this, SLOT(updateSelectionOnly()));
+    
     const Search::SearchOptions enabledOptions = searchOptions(SearchForward);
 
     const bool regexMode = enabledOptions.testFlag(Search::Regex);
@@ -865,6 +880,9 @@ int KateSearchBar::findAll(Range inputRange, const QString * replacement)
 
     delete workingRange;
 
+    // restore connection
+    connect(m_view, SIGNAL(selectionChanged(KTextEditor::View*)), this, SLOT(updateSelectionOnly()));
+    
     return matchCounter;
 }
 
@@ -890,7 +908,7 @@ void KateSearchBar::replaceAll()
     int replacementsDone=findAll(inputRange, &replacement);
 
     // send passive notification to view
-    showInfoMessage(i18np("1 replacement has been made", "%1 replacements have been made", replacementsDone));
+    showInfoMessage(i18ncp("short translation", "1 replacement made", "%1 replacements made", replacementsDone));
 
     // Never merge replace actions with other replace actions/user actions
     m_view->doc()->undoManager()->undoSafePoint();
