@@ -38,7 +38,6 @@
 #include "katehighlightmenu.h"
 #include "katedialogs.h"
 #include "katetextline.h"
-#include "katecodefolding.h"
 #include "kateschema.h"
 #include "katebookmarks.h"
 #include "kateconfig.h"
@@ -46,7 +45,7 @@
 #include "kateautoindent.h"
 #include "katecompletionwidget.h"
 #include "katesearchbar.h"
-#include "katevimodebar.h"
+#include "kateviemulatedcommandbar.h"
 #include "katepartpluginmanager.h"
 #include "katewordcompletion.h"
 #include "katelayoutcache.h"
@@ -113,8 +112,9 @@ KateView::KateView( KateDocument *doc, QWidget *parent )
     , m_annotationModel(0)
     , m_hasWrap( false )
     , m_doc( doc )
+    , m_textFolding (doc->buffer())
     , m_config( new KateViewConfig( this ) )
-    , m_renderer( new KateRenderer( doc, this ) )
+    , m_renderer( new KateRenderer( doc, m_textFolding, this ) )
     , m_viewInternal( new KateViewInternal( this ) )
     , m_spell( new KateSpellCheckDialog( this ) )
     , m_bookmarks( new KateBookmarks( this ) )
@@ -127,7 +127,7 @@ KateView::KateView( KateDocument *doc, QWidget *parent )
     , m_cmdLine (0)
     , m_console (0)
     , m_searchBar (0)
-    , m_viModeBar (0)
+    , m_viModeEmulatedCommandBar(0)
     , m_gotoBar (0)
     , m_dictionaryBar(NULL)
     , m_spellingMenu( new KateSpellingMenu( this ) )
@@ -135,6 +135,8 @@ KateView::KateView( KateDocument *doc, QWidget *parent )
     , m_delayedUpdateTriggered (false)
     , m_lineToUpdateMin (-1)
     , m_lineToUpdateMax (-1)
+    , m_floatTopMessageWidget (0)
+    , m_floatBottomMessageWidget (0)
 {
   // queued connect to collapse view updates for range changes, INIT THIS EARLY ENOUGH!
   connect(this, SIGNAL(delayedUpdateOfView()), this, SLOT(slotDelayedUpdateOfView()), Qt::QueuedConnection);
@@ -231,15 +233,10 @@ KateView::KateView( KateDocument *doc, QWidget *parent )
   else
     m_vBox->addWidget(m_bottomViewBar);
 
-  // add grid layout for floating message widget to ViewInternal
-  QGridLayout* gridLayout = new QGridLayout(m_viewInternal);
-  gridLayout->setContentsMargins(20, 20, 20, 20);
-  m_overlayMessageWidget = new KateMessageWidget(m_viewInternal, true);
-  m_overlayMessageWidget->hide();
-  gridLayout->setRowStretch(0, 1);
-  gridLayout->setColumnStretch(0, 1);
-  gridLayout->addWidget(m_overlayMessageWidget, 1, 1);
-  m_viewInternal->setLayout(gridLayout);
+  // add layout for floating message widgets to KateViewInternal
+  m_notificationLayout = new QVBoxLayout(m_viewInternal);
+  m_notificationLayout->setContentsMargins(20, 20, 20, 20);
+  m_viewInternal->setLayout(m_notificationLayout);
 
   // this really is needed :)
   m_viewInternal->updateView ();
@@ -276,20 +273,21 @@ KateView::KateView( KateDocument *doc, QWidget *parent )
   slotHlChanged();
   KCursor::setAutoHideCursor( m_viewInternal, true );
 
-  if ( viInputMode() && !config()->viInputModeHideStatusBar() ) {
+  if ( viInputMode() /* && FIXME: HAEHH? !config()->viInputModeHideStatusBar() */ ) {
     deactivateEditActions();
-    showViModeBar();
   }
 
   // user interaction (scrollling) starts notification auto-hide timer
   connect(this, SIGNAL(displayRangeChanged(KateView*)), m_topMessageWidget, SLOT(startAutoHideTimer()));
   connect(this, SIGNAL(displayRangeChanged(KateView*)), m_bottomMessageWidget, SLOT(startAutoHideTimer()));
-  connect(this, SIGNAL(displayRangeChanged(KateView*)), m_overlayMessageWidget, SLOT(startAutoHideTimer()));
 
   // user interaction (cursor navigation) starts notification auto-hide timer
   connect(this, SIGNAL(cursorPositionChanged(KTextEditor::View*, const KTextEditor::Cursor&)), m_topMessageWidget, SLOT(startAutoHideTimer()));
   connect(this, SIGNAL(cursorPositionChanged(KTextEditor::View*, const KTextEditor::Cursor&)), m_bottomMessageWidget, SLOT(startAutoHideTimer()));
-  connect(this, SIGNAL(cursorPositionChanged(KTextEditor::View*, const KTextEditor::Cursor&)), m_overlayMessageWidget, SLOT(startAutoHideTimer()));
+
+  // folding restoration on reload
+  connect(m_doc, SIGNAL(aboutToReload(KTextEditor::Document*)), SLOT(saveFoldingState()));
+  connect(m_doc, SIGNAL(reloaded(KTextEditor::Document*)), SLOT(applyFoldingState()));
 }
 
 KateView::~KateView()
@@ -358,10 +356,10 @@ void KateView::setupActions()
 
   m_copy = a = ac->addAction(KStandardAction::Copy, this, SLOT(copy()));
   a->setWhatsThis(i18n( "Use this command to copy the currently selected text to the system clipboard."));
-  
+
   m_pasteMenu = ac->addAction("edit_paste_menu", new KatePasteMenu (i18n("Clipboard &History"), this));
   connect (KateGlobal::self(), SIGNAL(clipboardHistoryChanged()), this, SLOT(slotClipboardHistoryChanged()));
-   
+
   if (!m_doc->readOnly())
   {
     a = ac->addAction(KStandardAction::Save, m_doc, SLOT(documentSave()));
@@ -1013,14 +1011,17 @@ void KateView::setupEditActions()
 
 void KateView::setupCodeFolding()
 {
+  //FIXME: FOLDING
   KActionCollection *ac=this->actionCollection();
 
-  KAction* a = ac->addAction("folding_toplevel");
+  KAction* a;
+
+  a = ac->addAction("folding_toplevel");
   a->setText(i18n("Fold Toplevel Nodes"));
   a->setShortcut(QKeySequence(Qt::CTRL+Qt::SHIFT+Qt::Key_Minus));
-  connect(a, SIGNAL(triggered(bool)), m_doc->foldingTree(), SLOT(collapseToplevelNodes()));
+  connect(a, SIGNAL(triggered(bool)), SLOT(slotFoldToplevelNodes()));
 
-  a = ac->addAction("folding_expandtoplevel");
+  /*a = ac->addAction("folding_expandtoplevel");
   a->setText(i18n("Unfold Toplevel Nodes"));
   a->setShortcut(QKeySequence(Qt::CTRL+Qt::SHIFT+Qt::Key_Plus));
   connect(a, SIGNAL(triggered(bool)), m_doc->foldingTree(), SLOT(expandToplevelNodes()));
@@ -1032,7 +1033,7 @@ void KateView::setupCodeFolding()
   a = ac->addAction("folding_collapse_dsComment");
   a->setText(i18n("Fold Multiline Comments"));
   connect(a, SIGNAL(triggered(bool)), m_doc->foldingTree(), SLOT(collapseAll_dsComments()));
-
+*/
   a = ac->addAction("folding_collapselocal");
   a->setText(i18n("Fold Current Node"));
   connect(a, SIGNAL(triggered(bool)), SLOT(slotCollapseLocal()));
@@ -1040,35 +1041,32 @@ void KateView::setupCodeFolding()
   a = ac->addAction("folding_expandlocal");
   a->setText(i18n("Unfold Current Node"));
   connect(a, SIGNAL(triggered(bool)), SLOT(slotExpandLocal()));
+}
 
-  // Explicit folding is allowed for level 2 to 4
-  for (int i = 2 ; i < 5 ; ++i) {
-    // Collapse level connections
-    a = ac->addAction(QString("collapse_level_%1").arg(i));
-    a->setText(i18n("Fold Nodes in Level %1", i));
-    a->setData(i);
-    connect(a, SIGNAL(triggered()), this, SLOT(slotCollapseLevel()));
-
-    // Expand level connections
-    a = ac->addAction(QString("expand_level_%1").arg(i));
-    a->setText(i18n("Unfold Nodes in Level %1", i));
-    a->setData(i);
-    connect(a, SIGNAL(triggered()), this, SLOT(slotExpandLevel()));
+void KateView::slotFoldToplevelNodes()
+{
+  // FIXME: more performant implementation
+  for (int line = 0; line < doc()->lines(); ++line) {
+    if (textFolding().isLineVisible(line)) {
+      foldLine(line);
+    }
   }
 }
 
 void KateView::slotCollapseLocal()
 {
-  m_doc->foldingTree()->collapseOne(cursorPosition().line(), cursorPosition().column());
+  foldLine (cursorPosition().line());
 }
 
 void KateView::slotExpandLocal()
 {
-  m_doc->foldingTree()->expandOne(cursorPosition().line(), cursorPosition().column());
+  unfoldLine (cursorPosition().line());
 }
 
 void KateView::slotCollapseLevel()
 {
+  //FIXME: FOLDING
+#if 0
   if (!sender()) return;
   QAction *action = qobject_cast<QAction*>(sender());
   if (!action) return;
@@ -1076,10 +1074,13 @@ void KateView::slotCollapseLevel()
   const int level = action->data().toInt();
   Q_ASSERT(level > 0);
   m_doc->foldingTree()->collapseLevel(level);
+#endif
 }
 
 void KateView::slotExpandLevel()
 {
+  //FIXME: FOLDING
+#if 0
   if (!sender()) return;
   QAction *action = qobject_cast<QAction*>(sender());
   if (!action) return;
@@ -1087,20 +1088,80 @@ void KateView::slotExpandLevel()
   const int level = action->data().toInt();
   Q_ASSERT(level > 0);
   m_doc->foldingTree()->expandLevel(level);
+#endif
+}
+
+void KateView::foldLine (int startLine)
+{
+  // only for valid lines
+  if (startLine < 0)
+    return;
+
+  // try to fold all known ranges
+  QVector<QPair<qint64, Kate::TextFolding::FoldingRangeFlags> > startingRanges = textFolding().foldingRangesStartingOnLine (startLine);
+  for (int i = 0; i < startingRanges.size(); ++i)
+    textFolding().foldRange (startingRanges[i].first);
+
+  // try if the highlighting can help us and create a fold
+  textFolding().newFoldingRange (doc()->buffer().computeFoldingRangeForStartLine (startLine), Kate::TextFolding::Folded);
+}
+
+void KateView::unfoldLine (int startLine)
+{
+  // only for valid lines
+  if (startLine < 0)
+    return;
+
+  // try to unfold all known ranges
+  QVector<QPair<qint64, Kate::TextFolding::FoldingRangeFlags> > startingRanges = textFolding().foldingRangesStartingOnLine (startLine);
+  for (int i = 0; i < startingRanges.size(); ++i)
+    textFolding().unfoldRange (startingRanges[i].first);
+}
+
+KTextEditor::View::EditMode KateView::viewEditMode() const
+{
+  if (viInputMode())
+    return EditViMode;
+  
+  return isOverwriteMode() ? EditOverwrite : EditInsert;
 }
 
 QString KateView::viewMode () const
 {
-  if (!m_doc->isReadWrite())
-    return i18n ("R/O");
-
+  /**
+   * normal two modes
+   */
+  QString currentMode = isOverwriteMode() ? i18n("OVR") : i18n ("INS");
+  
+  /**
+   * if we are in vi mode, this will be overwritten by current vi mode
+   */
   if (viInputMode()) {
-      // vi mode has different notion of OVR/INS.
-      // vi mode's status is shown in viModeBar() instead
-      return QString();
+    currentMode = KateViInputModeManager::modeToString (getCurrentViMode());
+    
+    /**
+     * perhaps append the current keys of a command not finalized
+     */
+    QString cmd = m_viewInternal->getViInputModeManager()->getVerbatimKeys();
+    if (!cmd.isEmpty())
+      currentMode.append (QString (" <em>%1</em>").arg (cmd));
+    
+    /**
+     * make it bold
+     */
+    currentMode = QString ("<b>%1</b>").arg (currentMode);
   }
-
-  return isOverwriteMode() ? i18n("OVR") : i18n ("INS");
+  
+  /**
+   * append read-only if needed
+   */
+  if (!m_doc->isReadWrite())
+    currentMode = i18n ("%1 (R/O)", currentMode);
+  
+  /**
+   * return full mode
+   */
+  return currentMode;
 }
 
 void KateView::slotGotFocus()
@@ -1152,7 +1213,7 @@ void KateView::slotReadWriteChanged ()
 
   QStringList l;
 
-  l << "edit_replace" << "set_insert" << "tools_spelling" << "tools_indent"
+  l << "edit_replace" << "tools_spelling" << "tools_indent"
       << "tools_unindent" << "tools_cleanIndent" << "tools_align"  << "tools_comment"
       << "tools_uncomment" << "tools_toggle_comment" << "tools_uppercase" << "tools_lowercase"
       << "tools_capitalize" << "tools_join_lines" << "tools_apply_wordwrap"
@@ -1164,10 +1225,14 @@ void KateView::slotReadWriteChanged ()
     if ((a = actionCollection()->action( l[z].toAscii().constData() )))
       a->setEnabled (m_doc->isReadWrite());
   slotUpdateUndo();
-  
+
   // inform search bar
   if (m_searchBar)
     m_searchBar->slotReadWriteChanged ();
+    
+  // => view mode changed
+  emit viewModeChanged(this);
+  emit viewEditModeChanged(this,viewEditMode());
 }
 
 void KateView::slotClipboardHistoryChanged ()
@@ -1446,14 +1511,8 @@ void KateView::toggleViInputMode()
 
   if ( viInputMode() ) {
     m_viewInternal->getViInputModeManager()->viEnterNormalMode();
-
-    if ( !config()->viInputModeHideStatusBar() ) {
-      showViModeBar();
-    }
-
     deactivateEditActions();
   } else { // disabling the vi input mode
-    hideViModeBar();
     activateEditActions();
   }
 
@@ -1461,36 +1520,26 @@ void KateView::toggleViInputMode()
   emit viewEditModeChanged(this,viewEditMode());
 }
 
-void KateView::showViModeBar()
+void KateView::showViModeEmulatedCommandBar()
 {
-  if (viInputMode() && !config()->viInputModeHideStatusBar()) {
-    bottomViewBar()->addPermanentBarWidget(viModeBar());
-    updateViModeBarMode();
-  }
-}
-
-void KateView::hideViModeBar()
-{
-  if (bottomViewBar() && m_viModeBar) {
-    bottomViewBar()->removePermanentBarWidget(viModeBar());
+  if (viInputMode() && config()->viInputModeEmulateCommandBar()) {
+    bottomViewBar()->addBarWidget(viModeEmulatedCommandBar());
+    bottomViewBar()->showBarWidget(viModeEmulatedCommandBar());
   }
 }
 
 void KateView::updateViModeBarMode()
-{
-  if (config()->viInputModeHideStatusBar())
-    return;
-
-  viModeBar()->updateViMode(getCurrentViMode());
+{  
+  // view mode changed => status bar in container apps might change!
+  emit viewModeChanged (this);
+  emit viewEditModeChanged(this,viewEditMode());
 }
 
 void KateView::updateViModeBarCmd()
 {
-  if (config()->viInputModeHideStatusBar())
-    return;
-
-  QString cmd = m_viewInternal->getViInputModeManager()->getVerbatimKeys();
-  viModeBar()->updatePartialCommand(cmd);
+  // view mode changed => status bar in container apps might change!
+  emit viewModeChanged (this);
+  emit viewEditModeChanged(this,viewEditMode());
 }
 
 ViMode KateView::getCurrentViMode() const
@@ -1573,7 +1622,6 @@ void KateView::switchToCmdLine ()
   }
   bottomViewBar()->showBarWidget(cmdLineBar());
   cmdLineBar()->setFocus ();
-  hideViModeBar();
 }
 
 void KateView::switchToConsole ()
@@ -1583,7 +1631,6 @@ void KateView::switchToConsole ()
   bottomViewBar()->addBarWidget(m_console);
   bottomViewBar()->showBarWidget(m_console);
   m_console->setFocus ();
-  hideViModeBar();
 }
 
 KateRenderer *KateView::renderer ()
@@ -1730,16 +1777,17 @@ void KateView::updateRendererConfig()
 
 // @@ showIndentLines is not cached anymore.
 //  m_renderer->setShowIndentLines (m_renderer->config()->showIndentationLines());
+  emit configChanged();
 }
 
 void KateView::updateFoldingConfig ()
 {
   // folding bar
-  bool doit = config()->foldingBar() && m_doc->highlight() && m_doc->highlight()->allowsFolding();
-  m_viewInternal->m_leftBorder->setFoldingMarkersOn(doit);
-  m_toggleFoldingMarkers->setChecked( doit );
-  m_toggleFoldingMarkers->setEnabled( m_doc->highlight() && m_doc->highlight()->allowsFolding() );
+  m_viewInternal->m_leftBorder->setFoldingMarkersOn(config()->foldingBar());
+  m_toggleFoldingMarkers->setChecked( config()->foldingBar() );
 
+#if 0
+  // FIXME: FOLDING
   QStringList l;
 
   l << "folding_toplevel" << "folding_expandtoplevel"
@@ -1749,6 +1797,7 @@ void KateView::updateFoldingConfig ()
   for (int z = 0; z < l.size(); z++)
     if ((a = actionCollection()->action( l[z].toAscii().constData() )))
       a->setEnabled (m_doc->highlight() && m_doc->highlight()->allowsFolding());
+#endif
 }
 
 void KateView::ensureCursorColumnValid()
@@ -2292,8 +2341,6 @@ void KateView::sendCompletionAborted()
 void KateView::paste(const QString *textToPaste)
 {
   m_doc->paste( this, textToPaste ? *textToPaste : QApplication::clipboard()->text(QClipboard::Clipboard) );
-  emit selectionChanged (this);
-  m_viewInternal->repaint();
 }
 
 void KateView::setCaretStyle( KateRenderer::caretStyles style, bool repaint )
@@ -2877,16 +2924,6 @@ KateSearchBar *KateView::searchBar (bool initHintAsPower)
   return m_searchBar;
 }
 
-KateViModeBar *KateView::viModeBar()
-{
-  if (!m_viModeBar) {
-    m_viModeBar = new KateViModeBar(this);
-    m_viModeBar->hide ();
-  }
-
-  return m_viModeBar;
-}
-
 KateGotoBar *KateView::gotoBar ()
 {
   if (!m_gotoBar) {
@@ -2907,6 +2944,16 @@ KateDictionaryBar *KateView::dictionaryBar ()
   return m_dictionaryBar;
 }
 
+KateViEmulatedCommandBar* KateView::viModeEmulatedCommandBar()
+{
+  if (!m_viModeEmulatedCommandBar) {
+    m_viModeEmulatedCommandBar = new KateViEmulatedCommandBar(this, this);
+    m_viModeEmulatedCommandBar->hide ();
+  }
+
+  return m_viModeEmulatedCommandBar;
+}
+
 void KateView::setAnnotationModel( KTextEditor::AnnotationModel* model )
 {
   KTextEditor::AnnotationModel* oldmodel = m_annotationModel;
@@ -3108,9 +3155,34 @@ void KateView::postMessage(KTextEditor::Message* message,
     m_topMessageWidget->postMessage(message, actions);
   } else if (message->position() == KTextEditor::Message::BelowView) {
     m_bottomMessageWidget->postMessage(message, actions);
-  } else {
-    m_overlayMessageWidget->postMessage(message, actions);
+  } else if (message->position() == KTextEditor::Message::TopInView) {
+    if (!m_floatTopMessageWidget) {
+      m_floatTopMessageWidget = new KateMessageWidget(m_viewInternal, true);
+      m_notificationLayout->insertWidget(0, m_floatTopMessageWidget, 0, Qt::AlignTop | Qt::AlignRight);
+      connect(this, SIGNAL(displayRangeChanged(KateView*)), m_floatTopMessageWidget, SLOT(startAutoHideTimer()));
+      connect(this, SIGNAL(cursorPositionChanged(KTextEditor::View*, const KTextEditor::Cursor&)), m_floatTopMessageWidget, SLOT(startAutoHideTimer()));
+    }
+    m_floatTopMessageWidget->postMessage(message, actions);
+  } else if (message->position() == KTextEditor::Message::BottomInView) {
+    if (!m_floatBottomMessageWidget) {
+      m_floatBottomMessageWidget = new KateMessageWidget(m_viewInternal, true);
+      m_notificationLayout->addWidget(m_floatBottomMessageWidget, 0, Qt::AlignBottom | Qt::AlignRight);
+      connect(this, SIGNAL(displayRangeChanged(KateView*)), m_floatBottomMessageWidget, SLOT(startAutoHideTimer()));
+      connect(this, SIGNAL(cursorPositionChanged(KTextEditor::View*, const KTextEditor::Cursor&)), m_floatBottomMessageWidget, SLOT(startAutoHideTimer()));
+    }
+    m_floatBottomMessageWidget->postMessage(message, actions);
   }
 }
 
+void KateView::saveFoldingState ()
+{
+  m_savedFoldingState = m_textFolding.exportFoldingRanges ();
+}
+    
+void KateView::applyFoldingState ()
+{
+  m_textFolding.importFoldingRanges (m_savedFoldingState);
+  m_savedFoldingState.clear ();
+}
+
 // kate: space-indent on; indent-width 2; replace-tabs on;
