@@ -34,7 +34,6 @@
 #include "kateconfig.h"
 #include "katedocument.h"
 #include "kateviewinternal.h"
-#include "katevimodebar.h"
 #include <ktexteditor/view.h>
 #include "katerenderer.h"
 
@@ -51,6 +50,13 @@ using KTextEditor::Range;
 // HELPER METHODS
 ////////////////////////////////////////////////////////////////////////////////
 
+void KateViModeBase::yankToClipBoard(QChar chosen_register, QString text)
+{
+ if ((chosen_register == '0' || chosen_register == '-')  && text.length() > 1) { //only yank to the clipboard if no register was specified and textlength > 1
+   KateGlobal::self()->copyToClipboard(text);
+ }
+}
+
 bool KateViModeBase::deleteRange( KateViRange &r, OperationMode mode, bool addToRegister)
 {
   r.normalize();
@@ -67,13 +73,16 @@ bool KateViModeBase::deleteRange( KateViRange &r, OperationMode mode, bool addTo
       res = doc()->removeText( Range( r.startLine, r.startColumn, r.endLine, r.endColumn), mode == Block );
   }
 
+  QChar chosenRegister = getChosenRegister( '0' );
   if ( addToRegister ) {
     if ( r.startLine == r.endLine ) {
-      fillRegister( getChosenRegister( '-' ), removedText, mode );
+      chosenRegister = getChosenRegister( '-' );
+      fillRegister( chosenRegister    , removedText, mode );
     } else {
-      fillRegister( getChosenRegister( '0' ), removedText, mode );
+      fillRegister(chosenRegister,  removedText, mode );
     }
   }
+  yankToClipBoard(chosenRegister, removedText);
 
   return res;
 }
@@ -134,6 +143,12 @@ const QChar KateViModeBase::getCharUnderCursor() const
 }
 
 const QString KateViModeBase::getWordUnderCursor() const
+{
+
+  return doc()->text( getWordRangeUnderCursor() );
+}
+
+const Range KateViModeBase::getWordRangeUnderCursor() const
 {
   Cursor c( m_view->cursorPosition() );
 
@@ -148,6 +163,10 @@ const QString KateViModeBase::getWordUnderCursor() const
     if ( c.column() > doc()->lineLength( c.line() ) ) {
       c.setColumn(0);
       c.setLine( c.line()+1 );
+      if (c.line() == doc()->lines())
+      {
+        return Range::invalid();
+      }
     }
 
     ch = doc()->character( c );
@@ -162,59 +181,131 @@ const QString KateViModeBase::getWordUnderCursor() const
   Cursor c2 = findWordEnd( c1.line(), c1.column()+i-1, true );
   c2.setColumn( c2.column()+1 );
 
-  return doc()->text( Range( c1, c2 ) );
+  return Range(c1, c2);
 }
 
-KateViRange KateViModeBase::findPattern( const QString &pattern, bool backwards, int count ) const
+Range KateViModeBase::findPattern(const QString& pattern, bool backwards, bool caseSensitive, const Cursor& startFrom, int count) const
 {
-  kDebug( 13070 ) << "searching for pattern \"" << pattern << "\", backwards = " << backwards
-    << ", count = " << count;
-  if ( pattern.isEmpty() ) {
-    return KateViRange();
-  }
-
-  Cursor c( m_view->cursorPosition() );
-
+  Cursor searchBegin = startFrom;
   KTextEditor::Search::SearchOptions flags = KTextEditor::Search::Regex;
 
   if ( backwards ) {
     flags |= KTextEditor::Search::Backwards;
   }
+  if (!caseSensitive)
+  {
+    flags |= KTextEditor::Search::CaseInsensitive;
+  }
+  Range finalMatch;
+  for (int i = 0; i < count; i++)
+  {
+    if (!backwards)
+    {
+      const KTextEditor::Range matchRange = m_view->doc()->searchText(KTextEditor::Range(Cursor(searchBegin.line(), searchBegin.column() + 1), m_view->doc()->documentEnd()), pattern, flags).first();
 
-  for ( int i = count; i > 0; i-- ) {
-    // prepare two ranges, one from start → cursor and one from cursor → end
-    Range r1 = Range( Cursor( 0,0 ), c );
-
-    // we want to search from current position + one
-    if ( c.column() < doc()->lineLength( c.line() ) ) {
-      c.setColumn( c.column()+1 );
-    } else if ( c.line() < doc()->lines() ) {
-      c.setColumn( 0 );
-      c.setLine( c.line()+1 );
+      if (matchRange.isValid())
+      {
+        finalMatch = matchRange;
+      }
+      else
+      {
+        // Wrap around.
+        const KTextEditor::Range wrappedMatchRange = m_view->doc()->searchText(KTextEditor::Range(m_view->doc()->documentRange().start(), m_view->doc()->documentEnd()), pattern, flags).first();
+        if (wrappedMatchRange.isValid())
+        {
+          finalMatch = wrappedMatchRange;
+        }
+        else
+        {
+          return Range::invalid();
+        }
+      }
     }
+    else
+    {
+      // Ok - this is trickier: we can't search in the range from doc start to searchBegin, because
+      // the match might extend *beyond* searchBegin.
+      // We could search through the entire document and then filter out only those matches that are
+      // after searchBegin, but it's more efficient to instead search from the start of the
+      // document until the beginning of the line after searchBegin, and then filter.
+      // Unfortunately, searchText doesn't necessarily turn up all matches (just the first one, sometimes)
+      // so we must repeatedly search in such a way that the previous match isn't found, until we either
+      // find no matches at all, or the first match that is before searchBegin.
+      Cursor newSearchBegin = Cursor(searchBegin.line(), m_view->doc()->lineLength(searchBegin.line()));
+      Range bestMatch = Range::invalid();
+      while (true)
+      {
+        QVector<Range> matchesUnfiltered = m_view->doc()->searchText(Range(newSearchBegin, m_view->doc()->documentRange().start()), pattern, flags);
+        kDebug(13070) << "matchesUnfiltered: " << matchesUnfiltered << " searchBegin: " << newSearchBegin;
 
-    Range r2 = Range( c, doc()->documentEnd() );
+        if (matchesUnfiltered.size() == 1 && !matchesUnfiltered.first().isValid())
+        {
+          break;
+        }
 
-    //kDebug( 13070 ) << "r1: " << r1;
-    //kDebug( 13070 ) << "r2: " << r2;
+        // After sorting, the last element in matchesUnfiltered is the last match position.
+        qSort(matchesUnfiltered);
 
-    //// see if we can find the term before the end of the document (opposite if backwards)
-    //kDebug( 13070 ) << "searching for " << pattern << " in " << (backwards ? "r1" : "r2") << " backwards = " << backwards;
-    Range result = doc()->searchText( backwards ? r1 : r2, pattern, flags ).first();
-    //kDebug( 13070 ) << "result: " << result;
+        QVector<Range> filteredMatches;
+        foreach(Range unfilteredMatch, matchesUnfiltered)
+        {
+          if (unfilteredMatch.start() < searchBegin)
+          {
+            filteredMatches.append(unfilteredMatch);
+          }
+        }
+        if (!filteredMatches.isEmpty())
+        {
+          // Want the latest matching range that is before searchBegin.
+          bestMatch = filteredMatches.last();
+          break;
+        }
 
-    if ( result.isValid() ) {
-      c = result.start();
-    } else {
+        // We found some unfiltered matches, but none were suitable. In case matchesUnfiltered wasn't
+        // all matching elements, search again, starting from before the earliest matching range.
+        if (filteredMatches.isEmpty())
+        {
+          newSearchBegin = matchesUnfiltered.first().start();
+        }
+      }
 
-      // no hits, continue from the top
-      result = doc()->searchText( backwards ? r2 : r1, pattern, flags ).first();
+      Range matchRange = bestMatch;
 
-      c = result.start();
+      if (matchRange.isValid())
+      {
+        finalMatch = matchRange;
+      }
+      else
+      {
+        const KTextEditor::Range wrappedMatchRange = m_view->doc()->searchText(KTextEditor::Range(m_view->doc()->documentEnd(), m_view->doc()->documentRange().start()), pattern, flags).first();
+
+
+        if (wrappedMatchRange.isValid())
+        {
+          finalMatch = wrappedMatchRange;
+        }
+        else
+        {
+          return Range::invalid();
+        }
+      }
     }
+    searchBegin = finalMatch.start();
+  }
+  return finalMatch;
+}
+
+KateViRange KateViModeBase::findPatternForMotion( const QString& pattern, bool backwards, bool caseSensitive, const Cursor& startFrom, int count ) const
+{
+  kDebug( 13070 ) << "searching for pattern \"" << pattern << "\", backwards = " << backwards
+    << ", caseSensitive = " << caseSensitive << ", count = " << count;
+  if ( pattern.isEmpty() ) {
+    return KateViRange();
   }
 
-  return KateViRange( c.line(), c.column(), ViMotion::ExclusiveMotion );
+  Range match = findPattern(pattern, backwards, caseSensitive, startFrom, count);
+
+  return KateViRange( match.start().line(), match.start().column(), ViMotion::ExclusiveMotion );
 }
 
 Cursor KateViModeBase::findNextWordStart( int fromLine, int fromColumn, bool onlyCurrentLine ) const
@@ -590,10 +681,14 @@ KateViRange innerRange(KateViRange range, bool inner) {
   KateViRange r = range;
 
   if (inner) {
+    const int columnDistance = qAbs(r.startColumn - r.endColumn);
+    if ((r.startLine == r.endLine) && columnDistance == 1 )
+    {
+      // Start and end are right next to each other; there is nothing inside them.
+      r.valid = false;
+    }
     r.startColumn++;
     r.endColumn--;
-    if ((r.startLine == r.endLine) && qAbs(r.startColumn - r.endColumn) < 2 )
-      r.valid = false;
   }
 
   return r;
@@ -816,7 +911,7 @@ int KateViModeBase::findLineStartingWitchChar( const QChar &c, unsigned int coun
     line--;
   }
 
-  while ( line < lines && line > 0 && hits < count ) {
+  while ( line < lines && line >= 0 && hits < count ) {
     QString l = getLine( line );
     if ( l.length() > 0 && l.at( 0 ) == c ) {
       hits++;
@@ -852,7 +947,7 @@ QChar KateViModeBase::getChosenRegister( const QChar &defaultReg ) const
   return reg;
 }
 
-QString KateViModeBase::getRegisterContent( const QChar &reg ) const
+QString KateViModeBase::getRegisterContent( const QChar &reg )
 {
   QString r = KateGlobal::self()->viInputModeGlobal()->getRegisterContent( reg );
 
@@ -979,20 +1074,46 @@ KateViRange KateViModeBase::goVisualLineUpDown(int lines) {
   if (r.endColumn == -1 || r.endLine == -1 || r.valid == false)
     return r;
 
-  Kate::TextLine startLine = doc()->plainKateTextLine( c.line() );
-  Kate::TextLine endLine = doc()->plainKateTextLine( r.endLine );
-
-  int virtColumnStart = startLine->toVirtualColumn(c.column(), tabstop);
-
-  int view_line = m_viewInternal->cache()->viewLine(c);
-  int visualColumnStart = virtColumnStart - m_viewInternal->cache()->viewLine(view_line).startCol();
-
-  if ( m_stickyColumn == -1 ) {
-     m_stickyColumn = visualColumnStart;
+  if ( m_stickyColumn == -1) {
+    // Compute new sticky column.
+    const Kate::TextLine startLine = doc()->plainKateTextLine( c.line() );
+    const int virtColumnStart = startLine->toVirtualColumn(c.column(), tabstop);
+    const int visualColumnStart = virtColumnStart - m_viewInternal->cache()->textLayout(c).startCol();
+    //Q_ASSERT(m_stickyColumn >= 0);
+    m_stickyColumn = visualColumnStart;
+    Q_ASSERT(m_stickyColumn >= 0);
   } else {
-    int end_view_line = m_viewInternal->cache()->viewLine(res);
-    int s_c = m_viewInternal->cache()->textLayout(res.line(), end_view_line).startCol();
-    r.endColumn = endLine->fromVirtualColumn( m_stickyColumn + s_c, tabstop );
+    const int endViewLine = m_viewInternal->cache()->viewLine(res);
+    // The "real" (non-virtual) beginning of the current "line", which might be a wrapped continuation of a
+    // "real" line.
+    const int realLineStartColumn = m_viewInternal->cache()->textLayout(res.line(), endViewLine).startCol();
+    const Kate::TextLine endLine = doc()->plainKateTextLine( r.endLine );
+    // Adjust for the fact that if the portion of the line before wrapping is indented,
+    // the continuations are also "invisibly" (i.e. without any spaces in the text itself) indented.
+    const bool isWrappedContinuation = (m_viewInternal->cache()->textLayout(res.line(), endViewLine).lineLayout().lineNumber() != 0);
+    const int numInvisibleIndentChars = isWrappedContinuation ? endLine->toVirtualColumn(m_viewInternal->cache()->line(res.line())->textLine()->nextNonSpaceChar(0), tabstop) : 0;
+    if (m_stickyColumn == (unsigned int)KateVi::EOL)
+    {
+      const int visualEndColumn = m_viewInternal->cache()->textLayout(res.line(), endViewLine).lineLayout().textLength() - 1;
+      r.endColumn = endLine->fromVirtualColumn( visualEndColumn + realLineStartColumn - numInvisibleIndentChars, tabstop );
+    }
+    else
+    {
+      // Algorithm: find the "real" column corresponding to the start of the line.  Offset from that
+      // until the "visual" column is equal to the "visual" sticky column.
+      int realOffsetToVisualStickyColumn = 0;
+      const int lineStartVirtualColumn = endLine->toVirtualColumn( realLineStartColumn, tabstop );
+      while (true)
+      {
+        const int visualColumn = endLine->toVirtualColumn( realLineStartColumn + realOffsetToVisualStickyColumn, tabstop ) - lineStartVirtualColumn + numInvisibleIndentChars;
+        if (visualColumn >= m_stickyColumn)
+        {
+          break;
+        }
+        realOffsetToVisualStickyColumn++;
+      }
+      r.endColumn = realLineStartColumn + realOffsetToVisualStickyColumn;
+    }
   }
 
   return r;
@@ -1078,14 +1199,36 @@ bool KateViModeBase::startVisualLineMode()
   return true;
 }
 
-void KateViModeBase::error( const QString &errorMsg ) const
+void KateViModeBase::error( const QString &errorMsg )
 {
-  m_view->viModeBar()->showErrorMessage(errorMsg);
+  delete m_infoMessage;
+
+  // nop if no vi mode around
+  if (!m_view->viInputMode())
+    return;
+
+  m_infoMessage = new KTextEditor::Message(KTextEditor::Message::Error, errorMsg);
+  m_infoMessage->setPosition(KTextEditor::Message::BottomInView);
+  m_infoMessage->setAutoHide(2000); // 2 seconds
+  m_infoMessage->setView(m_view);
+
+  m_view->doc()->postMessage(m_infoMessage);
 }
 
-void KateViModeBase::message( const QString &msg ) const
+void KateViModeBase::message( const QString &msg )
 {
-  m_view->viModeBar()->showMessage(msg);
+  delete m_infoMessage;
+
+  // nop if no vi mode around
+  if (!m_view->viInputMode())
+    return;
+
+  m_infoMessage = new KTextEditor::Message(KTextEditor::Message::Positive, msg);
+  m_infoMessage->setPosition(KTextEditor::Message::BottomInView);
+  m_infoMessage->setAutoHide(2000); // 2 seconds
+  m_infoMessage->setView(m_view);
+
+  m_view->doc()->postMessage(m_infoMessage);
 }
 
 QString KateViModeBase::getVerbatimKeys() const
@@ -1131,41 +1274,76 @@ void KateViModeBase::addToNumberUnderCursor( int count )
     Cursor c( m_view->cursorPosition() );
     QString line = getLine();
 
+    if (line.isEmpty())
+    {
+      return;
+    }
+
     int wordStart = findPrevWordStart( c.line(), c.column()+1, true ).column();
-    int wordEnd = findWordEnd( c.line(), c.column()-1, true ).column();
 
     if (wordStart > 0 && line.at(wordStart - 1) == '-') wordStart--;
 
     QRegExp number( "(0x)([0-9a-fA-F]+)|\\-?\\d+" );
 
     int start = number.indexIn( line, wordStart );
-    if ( start <= wordEnd ) {
-        // FIXME: ignore leading zeroes
-        QString nString = number.cap();
-        bool ok = false;
-        int base = number.cap( 1 ).isEmpty() ? 10 : 16;
-        int n = nString.toInt( &ok, base );
 
-        kDebug( 13070 ) << "base: " << base;
-        kDebug( 13070 ) << "n: " << n;
-
-        if ( !ok ) {
-            // conversion to int failed. give up.
-            return;
-        }
-
-        // increase/decrease number
-        n += count;
-
-        // create the new text string to be inserted. prepend with “0x” if in base 16
-        QString newText = (base == 16 ? "0x" : "") + QString::number(n, base);
-
-        // replace the old number string with the new
-        doc()->editStart();
-        doc()->removeText( KTextEditor::Range( c.line(), start , c.line(), start+nString.length() ) );
-        doc()->insertText( KTextEditor::Cursor( c.line(), start ), newText );
-        doc()->editEnd();
+    QString nString = number.cap();
+    bool ok = false;
+    int base = number.cap( 1 ).isEmpty() ? 10 : 16;
+    if (base != 16 && nString.startsWith("0") && nString != "0")
+    {
+      nString.toInt( &ok, 8 );
+      if (ok)
+      {
+        // Octal.
+        base = 8;
+      }
     }
+    int n = nString.toInt( &ok, base );
+    QString withoutBase;
+    if (base == 16)
+    {
+      withoutBase = number.cap(2);
+    }
+    else if (base == 8)
+    {
+      withoutBase = nString.mid(1); // Strip off leading 0.
+    }
+    else
+    {
+      withoutBase = nString;
+    }
+
+    kDebug( 13070 ) << "base: " << base;
+    kDebug( 13070 ) << "n: " << n;
+
+    if ( !ok ) {
+        // conversion to int failed. give up.
+        return;
+    }
+
+    // increase/decrease number
+    n += count;
+
+    // create the new text string to be inserted. prepend with “0x” if in base 16, and "0" if base 8.
+    // For non-decimal numbers, try to keep the length of the number the same (including leading 0's).
+    QString newNumberPadded = (base == 16 || base == 8) ? QString("%1").arg(n, withoutBase.length(), base, QChar('0')) : QString("%1").arg(n, 0, base);
+    QString newText = newNumberPadded;
+    if (base == 16)
+    {
+      newText = "0x" + newText;
+    }
+    else if (base == 8)
+    {
+      newText = "0" + newText;
+    }
+
+    // replace the old number string with the new
+    doc()->editStart();
+    doc()->removeText( KTextEditor::Range( c.line(), start , c.line(), start+nString.length() ) );
+    doc()->insertText( KTextEditor::Cursor( c.line(), start ), newText );
+    doc()->editEnd();
+    updateCursor(Cursor(m_view->cursorPosition().line(), start + newText.length() - 1));
 }
 
 void KateViModeBase::switchView(Direction direction) {
