@@ -97,11 +97,11 @@ class KATEPART_TESTS_EXPORT KateViNormalMode : public KateViModeBase
     bool commandYankLine();
     bool commandYankToEOL();
 
-    bool commandPasteLeaveCursorAtStart();
-    bool commandPasteBeforeLeaveCursorAtStart();
+    bool commandPaste();
+    bool commandPasteBefore();
 
-    bool commandPasteLeaveCursorAtEnd();
-    bool commandPasteBeforeLeaveCursorAtEnd();
+    bool commandgPaste();
+    bool commandgPasteBefore();
 
     bool commandDeleteChar();
     bool commandDeleteCharBackward();
@@ -265,12 +265,15 @@ class KATEPART_TESTS_EXPORT KateViNormalMode : public KateViModeBase
 
     void addCurrentPositionToJumpList();
 
-    void addMapping( const QString &from, const QString &to );
+    void addMapping( const QString& from, const QString& to, KateViModeBase::MappingRecursion recursion );
     const QString getMapping( const QString &from ) const;
     const QStringList getMappings() const;
+    bool isMappingRecursive(const QString& from) const;
     virtual void reset();
 
     void setMappingTimeout(int timeoutMS);
+
+    void beginMonitoringDocumentChanges();
   protected:
     void resetParser();
     void initializeCommands();
@@ -278,8 +281,10 @@ class KATEPART_TESTS_EXPORT KateViNormalMode : public KateViModeBase
     virtual void goToPos( const KateViRange &r );
     void executeCommand( const KateViCommand* cmd );
     OperationMode getOperationMode() const;
-    bool paste(bool leaveCursorAtStart);
-    bool pasteBefore(bool leaveCursorAtStart);
+    // The 'current position' is the current cursor position for non-linewise pastes, and the current
+    // line for linewise.
+    enum PasteLocation { AtCurrentPosition, AfterCurrentPosition };
+    bool paste(PasteLocation pasteLocation, bool isgPaste);
     Cursor cursorPosAtEndOfPaste(const Cursor& pasteLocation, const QString& pastedText);
 
     void joinLines(unsigned int from, unsigned int to) const;
@@ -297,6 +302,7 @@ class KATEPART_TESTS_EXPORT KateViNormalMode : public KateViModeBase
     QVector<int> m_matchingCommands;
     QVector<int> m_matchingMotions;
     QStack<int> m_awaitingMotionOrTextObject;
+    bool motionWillBeUsedWithCommand() { return !m_awaitingMotionOrTextObject.isEmpty(); };
 
     int m_motionOperatorIndex;
 
@@ -305,6 +311,8 @@ class KATEPART_TESTS_EXPORT KateViNormalMode : public KateViModeBase
 
     bool m_linewiseCommand;
     bool m_commandWithMotion;
+    bool m_isCommandBeingExecuted;
+    bool m_lastMotionWasLinewiseInnerBlock;
 
     bool m_commandShouldKeepSelection;
 
@@ -330,14 +338,34 @@ class KATEPART_TESTS_EXPORT KateViNormalMode : public KateViModeBase
     // "'123", and to choose between them.)
     QString m_fullMappingMatch;
     // set after f/F/t/T/r so the following character isn't translated
-    bool m_ignoreMapping;
+    bool m_doNotMapNextKeyPress;
     QString m_mappingKeys;
+    bool m_doNotExpandFurtherMappings;
     void executeMapping();
 
     KateViKeyParser *m_keyParser;
 
     // Ctrl-c or ESC have been pressed, leading to a call to reset().
     bool m_pendingResetIsDueToExit;
+
+    KTextEditor::Attribute::Ptr m_highlightYankAttribute;
+    KTextEditor::MovingRange* m_highlightedYank;
+
+    void highlightYank(const KateViRange& range);
+    KTextEditor::MovingRange*& highlightedYankForDocument();
+
+    Cursor m_currentChangeEndMarker;
+
+    bool m_isUndo;
+private slots:
+    void textInserted(KTextEditor::Document* document, KTextEditor::Range range);
+    void textRemoved(KTextEditor::Document*,KTextEditor::Range);
+    void undoBeginning();
+    void undoEnded();
+
+    void updateYankHighlightAttrib();
+    void clearYankHighlight();
+    void aboutToDeleteMovingInterfaceContent();
 };
 
 #endif
